﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;
using Infragistics.Windows.Editors;
using System.Reflection;
using System.ComponentModel;

namespace InplaceErrorPosition
{
    /// <summary>
    /// Interaction logic for InplaceErrorPosition.xaml
    /// </summary>
    public partial class IEPWindow : Window
    {
        #region Enums
        /// <summary>
        /// Enumeration of Various Modes
        /// </summary>
        [Flags]
        public enum Modes
        {
            /// <summary>
            /// Add = 2
            /// </summary>
            Add = 2,
            /// <summary>
            /// Edit = 8
            /// </summary>
            Edit = 8,
            /// <summary>
            /// Default = 4
            /// </summary>
            Default = 4,
            /// <summary>
            ///   None = 0
            /// </summary>
            None = 0,
            /// <summary>
            ///  Grouped = 32
            /// </summary>
            Grouped = 32,
            /// <summary>
            ///  RowActivated = 16
            /// </summary>
            RowActivated = 16,
            /// <summary>
            ///  Copy = 64
            /// </summary>
            Copy = 64
        }
        #endregion

        #region Private variables
        XamTextEditor editor;
        private Modes _userMode = Modes.Default;
        private Complexity _addedComplexity;
        object OrginalcomplexityData;
        #endregion

        #region Public properties
        public Modes UserMode
        {
            get { return _userMode; }
            set
            {
                _userMode = value;
                EnableDisableButtonBar();
            }
        }

        public bool IsDataChanged { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public String CommandDescription
        {
            get { return base.GetValue(CommandDescriptionProperty) as String; }
            set { base.SetValue(CommandDescriptionProperty, value); }
        }

        /// <summary>
        /// 
        /// </summary>
        public static readonly DependencyProperty CommandDescriptionProperty = DependencyProperty.Register("CommandDescription", typeof(String), typeof(Button));

        #endregion
        
        #region Constructors

        public IEPWindow()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(ComplexityView_Loaded);
            grdComplexity.PreviewKeyDown += new KeyEventHandler(grdComplexity_PreviewKeyDown);
        }


        private int GetSelectedItemCount()
        {
            if (grdComplexity.ActiveRecord != null && grdComplexity.ActiveRecord.RecordType != RecordType.GroupByField)
                return grdComplexity.SelectedItems.Records.Count;

            return 0;
        }

        #endregion

        #region Private Methods

        void ComplexityView_KeyUp(object sender, KeyEventArgs e)
        {
            if (!this.IsLoaded)
                return;
        }

        void grdComplexity_EditModeStarting(object sender, EditModeStartingEventArgs e)
        {
            ValueConstraint valueConstraint = new ValueConstraint();
            valueConstraint.MaxLength = 45;
            e.Editor.ValueConstraint = valueConstraint;
            e.Editor.TextChanged += new RoutedPropertyChangedEventHandler<string>(Editor_TextChanged);
        }

        void Editor_TextChanged(object sender, RoutedPropertyChangedEventArgs<string> e)
        {
            
        }

        void ComplexityView_Loaded(object sender, RoutedEventArgs e)
        {           
            EntityCollection<Complexity> complexitylist = new EntityCollection<Complexity>();
            Complexity _complexity1 = new Complexity();
            Complexity _complexity2 = new Complexity();
            Complexity _complexity3 = new Complexity();
            Complexity _complexity4 = new Complexity();
            _complexity1.Name = "Easy";
            _complexity1.DisplayOrder = 1;
            _complexity2.Name = "Moderate";
            _complexity2.DisplayOrder = 2;
            _complexity3.Name = "Intricate";
            _complexity3.DisplayOrder = 3;
            _complexity4.Name = "Defficult";
            _complexity4.DisplayOrder = 4;
            complexitylist.Add(_complexity1);
            complexitylist.Add(_complexity2);
            complexitylist.Add(_complexity3);
            complexitylist.Add(_complexity4);
            BindData(complexitylist);
        }


        void grdComplexity_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            Key key = e.Key;

            // We want to handle Alt key. Get the real key if it is Key.System.
            if (key == Key.System)
            {
                key = e.SystemKey;
            }

            switch (key)
            {                
                case Key.Return:
                    if ((UserMode & Modes.Edit) == Modes.Edit || (UserMode & Modes.Add) == Modes.Add)
                    {

                        if (SaveData() == false)
                            e.Handled = true;
                        else
                            IsDataChanged = false;
                    }
                    break;
                default:
                    e.Handled = false;
                    break;
            }
        }

        private bool SaveData()
        {
            try
            {
                EntityCollection<Complexity> updatedData=new EntityCollection<Complexity>();
                if ((UserMode & Modes.Add) == Modes.Add)
                {
                    updatedData = UpdatedRow();
                    Complexity newComplexity = NewlyAddedRow();
                    EntityCollection<Complexity> complexities = new EntityCollection<Complexity>();
                    complexities = (EntityCollection<Complexity>)grdComplexity.DataSource as EntityCollection<Complexity>;
                    if (!CheckForDuplicateExiest(complexities, newComplexity))
                    {
                        complexities.Add(newComplexity);
                        BindData(complexities);
                    }
                    else
                        throw new Exception("Duplicate Record");
                }
                else if ((UserMode & Modes.Edit) == Modes.Edit)
                {
                    updatedData = UpdatedRow();
                }

                ClearInplaceErrors();
                Clear();
                return true;
            }
            catch
            {                
            }
            CatchAndDisplayInplaceError("Duplicate Record");
            return false;
        }

        bool CheckForDuplicateExiest(EntityCollection<Complexity> updatedData, Complexity NewlyAddedRow)
         {
             foreach (Complexity cp in updatedData)
             {
                 if(cp.Name.ToString().Equals(NewlyAddedRow.Name.ToString()))
                     return true;
             }
             return false;
         }

        void btnNewClicked(object sender, RoutedEventArgs e)
        {
            UnboundField objField = new UnboundField() { ActualPosition = new FieldPosition(0, 0, 0, 0) };
            foreach (var item in grdComplexity.FieldLayouts[0].Fields)
            {
                item.Column++;
            }
            grdComplexity.FieldLayouts[0].Fields.Add(objField);

        }

        void btnEditClicked(object sender, RoutedEventArgs e)
        {
            EditRow();
        }
               
        private void SetDefaultData()
        {
            FieldLayout fl = grdComplexity.FieldLayouts[0];
            FieldSortDescription fsd = new FieldSortDescription();
            fsd.FieldName = "DisplayOrder";
            fsd.Direction = System.ComponentModel.ListSortDirection.Ascending;
            fl.SortedFields.Clear();
            fl.SortedFields.Add(fsd);
        }

        private void SetDataSource(List<Complexity> complexityData)
        {
            grdComplexity.DataSource = null;
            grdComplexity.DataSource = complexityData;
        }
                
        private void SetActivateRowBasedOnRecordCount()
        {
            if (grdComplexity.Records.Count != 0)
                grdComplexity.ActiveRecord = grdComplexity.Records[0];
        }
        
        private void EnableDisableButtonBar()
        {
            bool rowSelected = grdComplexity.ActiveRecord != null; ;

            if (rowSelected)
            {
                this.EnableButtonBarOnRowSelected();
            }

            else
            {
                this.EnableOnDefaultButtonBar();
            }
        }
                   
        private void EnableButtonBarOnRowSelected()
        {
                btnEdit.IsEnabled = true;
        }

        private void EnableOnDefaultButtonBar()
        {
            btnEdit.IsEnabled = false;
        }

        #endregion

        #region Public Methods
        
        public void SelectAllMenuItemSelect(object sender, RoutedEventArgs e)
        {
            if (grdComplexity.ActiveRecord == null && grdComplexity.Records.Count > 0)
                grdComplexity.Records[grdComplexity.Records.Count - 1].IsActive = true;
            for (int i = 0; i < grdComplexity.Records.Count; i++)
            {
                grdComplexity.Records[i].IsSelected = true;
            }
        }

        public void CatchAndDisplayInplaceError(string message)
        {
            try
            {
                if (!string.IsNullOrEmpty(message))
                {
                    List<FaultRecords> faultRecordsList = new List<FaultRecords>();
                    Record record = grdComplexity.ActiveRecord as Record;
                    FaultRecords faultRecords = new FaultRecords(record, message);            
                    faultRecordsList.Add(faultRecords);
                    if (grdComplexity.RecordManager.CurrentAddRecord != null)
                    {
                        this._userMode = Modes.Add;
                        ShowInplaceErrorWithNewRecord(faultRecordsList, faultRecordsList[0].FaultMessage);
                    }
                    else
                    {
                        ShowInplaceError(faultRecordsList);
                    }

                }
                else
                {
                    ClearInplaceError();
                }
            }
            catch
            {
               
            }
        }

        public void ShowInplaceError(List<FaultRecords> faultRecords)
        {
            ShowInplaceError(faultRecords, true, "");
        }

        public void ShowInplaceErrorWithNewRecord(List<FaultRecords> faultRecords, string newRecordErrorMessage)
        {
            ShowInplaceError(faultRecords, false, newRecordErrorMessage);
        }

        private void ShowInplaceError(List<FaultRecords> faultRecords, bool dontShowonNewRecord, string newrecorderrormessage)
        {
            if (faultRecords != null)
            {
                UnboundField objField = new UnboundField();

                if (grdComplexity.FieldLayouts[0].Fields[0].Name == "FaultRecord")
                {
                    objField = grdComplexity.FieldLayouts[0].Fields[0] as UnboundField;
                }
                else
                {
                    objField.Name = "FaultRecord";
                    objField.DataType = typeof(FaultRecords);
                    objField.Settings.AllowResize = false;
                    objField.Settings.AllowEdit = false;
                    objField.Label = "";
                    objField.Visibility = Visibility.Visible;
                    objField.Settings.LabelMaxWidth = 20;
                    objField.Settings.CellMaxWidth = 20;
                    grdComplexity.FieldLayouts.Add(new FieldLayout());
                    grdComplexity.FieldLayouts[0].Fields.Insert(0, objField);
                }
                foreach (Record gridrecord in grdComplexity.Records)
                {
                    if (gridrecord is DataRecord)
                    {
                        ((DataRecord)gridrecord).Cells[0].Value = new FaultRecords(true, "");
                        for (int i = 0; i < faultRecords.Count; i++)
                        {
                            if (gridrecord == faultRecords[i].ErrorRecord as Record)
                            {
                                ((DataRecord)gridrecord).Cells[0].Value = new FaultRecords(false, faultRecords[i].FaultMessage);
                            }

                        }
                    }
                }
                if (grdComplexity.RecordManager.CurrentAddRecord != null)
                {
                    grdComplexity.RecordManager.CurrentAddRecord.Cells[0].Value = new FaultRecords(dontShowonNewRecord, newrecorderrormessage);
                }
                ResourceDictionary resourceDictionary = new ResourceDictionary();
                Style validationErrorStyle = grdComplexity.FindResource("DataGridValidationFailedStyle") as Style;
                if (validationErrorStyle != null)
                {
                    objField.Settings.CellValuePresenterStyle = validationErrorStyle;
                }

                grdComplexity.Resources.MergedDictionaries.Remove(resourceDictionary);
            }
        }

        public void ClearInplaceError()
        {
            UnboundField objField = new UnboundField();
            if (grdComplexity.FieldLayouts[0].Fields[0].Name == "FaultRecord")
            {
                objField = grdComplexity.FieldLayouts[0].Fields[0] as UnboundField;
                grdComplexity.FieldLayouts[0].Fields.Remove(objField);
            }
        }
        #region IComplexityView Members

        
        public int NoOfRecordsInGrid()
        {
            return ((EntityCollection<Complexity>)grdComplexity.DataSource).Count;
        }

       
        public void BindData(EntityCollection<Complexity> complexityData)
        {
            grdComplexity.FieldSettings.AllowEdit = true;
            this.SetDataSource(complexityData);
            this.SetActivateRowBasedOnRecordCount();
            OrginalcomplexityData = (grdComplexity.DataSource as EntityCollection<Complexity>).Clone() as EntityCollection<Complexity>;
            grdComplexity.FieldSettings.CellClickAction = CellClickAction.Default;
        }

        public bool AddNewRow()
        {
            grdComplexity.FieldLayoutSettings.AddNewRecordLocation = AddNewRecordLocation.OnTop;
            grdComplexity.FieldLayoutSettings.AllowAddNew = true;
            grdComplexity.FieldSettings.AllowEdit = true;
            grdComplexity.RecordManager.CurrentAddRecord.Cells["Name"].Value = null;
            grdComplexity.ActiveCell = grdComplexity.RecordManager.CurrentAddRecord.Cells["Name"];
            grdComplexity.FieldSettings.CellClickAction = CellClickAction.Default;
            if (grdComplexity.RecordManager.CurrentAddRecord == null) return false;
            grdComplexity.ActiveRecord = grdComplexity.RecordManager.CurrentAddRecord;
            grdComplexity.RecordManager.CurrentAddRecord.Cells["DisplayOrder"].Value = GeMaxOrder() + 1;
            return true;
        }

        private UInt32 GeMaxOrder()
        {
            EntityCollection<Complexity> complexities = new EntityCollection<Complexity>();
            complexities = (EntityCollection<Complexity>)grdComplexity.DataSource as EntityCollection<Complexity>;
            return complexities.Count > 0 ? complexities.Max(complexity => complexity.DisplayOrder) : 0;
        }

        public bool StartEditNewRow()
        {
            editor = new XamTextEditor();
            grdComplexity.BringCellIntoView(grdComplexity.ActiveCell);
            DataPresenterCommands.StartEditMode.Execute(editor, grdComplexity);
            return true;
        }

        public bool EditRow()
        {
            if (grdComplexity.ActiveRecord == null) return false;
            grdComplexity.ActiveCell = ((DataRecord)grdComplexity.ActiveRecord).Cells["Name"];
            grdComplexity.FieldSettings.AllowEdit = true;
            grdComplexity.FieldSettings.CellClickAction = CellClickAction.Default;
            this.grdComplexity.FieldLayouts[0].Fields["DisplayOrder"].Settings.CellWidth = this.grdComplexity.FieldLayouts[0].Fields["DisplayOrder"].CellWidthResolved;
            editor = new XamTextEditor();
            grdComplexity.BringCellIntoView(grdComplexity.ActiveCell);
            DataPresenterCommands.StartEditMode.Execute(editor, grdComplexity);
            return true;
        }

        public void Clear()
        {
            UserMode = Modes.Default;
            grdComplexity.FieldLayoutSettings.AllowAddNew = false;
            grdComplexity.FieldSettings.CellClickAction = CellClickAction.Default;
            grdComplexity.ActiveRecord = null;
        }

        public void ClearInplaceErrors()
        {
            ClearInplaceError();
        }

        public Complexity NewlyAddedRow()
        {
            DataPresenterCommands.EndEditModeAndAcceptChanges.Execute(editor, grdComplexity);
            if (grdComplexity.RecordManager.CurrentAddRecord != null)
            {
                Complexity modifiedComplexity = ((grdComplexity.ActiveRecord as DataRecord).DataItem as Complexity);
                if (modifiedComplexity == null)
                {
                    Complexity complexity = new Complexity();
                    complexity.DisplayOrder = GeMaxOrder() + 1;
                    complexity.UniqueIdentifier = Guid.NewGuid();
                    _addedComplexity = modifiedComplexity;
                    return complexity;
                }
                else
                {
                    if (modifiedComplexity.UniqueIdentifier == Guid.Empty)
                        modifiedComplexity.UniqueIdentifier = Guid.NewGuid();

                    _addedComplexity = modifiedComplexity;
                    return modifiedComplexity;
                }
            }
            else
            {
                throw new Exception();
            }
        }

        public EntityCollection<Complexity> UpdatedRow()
        {
            DataPresenterCommands.EndEditModeAndAcceptChanges.Execute(editor, grdComplexity);
            EntityCollection<Complexity> dataItems = (grdComplexity.DataSource as EntityCollection<Complexity>).Clone() as EntityCollection<Complexity>;
            int totalRecordsCount = dataItems.Count;
            Complexity modifiedComplexity;

            if (UserMode == Modes.Add)
            {
                EntityCollection<Complexity> updateList = new EntityCollection<Complexity>();
                modifiedComplexity = ((grdComplexity.ActiveRecord as DataRecord).DataItem as Complexity);
                if (modifiedComplexity == null) return updateList;

                if (modifiedComplexity.UniqueIdentifier == Guid.Empty)
                    modifiedComplexity.UniqueIdentifier = Guid.NewGuid();

                if (modifiedComplexity.DisplayOrder < 1)
                {
                    updateList.Add(modifiedComplexity);
                    return updateList;
                }

                int newDisplayOrder = Convert.ToInt32(modifiedComplexity.DisplayOrder);
                if (newDisplayOrder > 0)
                {
                    foreach (Complexity complexity in dataItems)
                    {
                        if (complexity.DisplayOrder >= newDisplayOrder)
                        {
                            complexity.DisplayOrder += 1;
                            updateList.Add(complexity);
                        }
                    }
                }
                return updateList;
            }
            else
            {
                EntityCollection<Complexity> updatedItems = new EntityCollection<Complexity>();

                // get the modified item.
                modifiedComplexity = ((DataRecord)grdComplexity.ActiveRecord).DataItem as Complexity;
                _addedComplexity = modifiedComplexity;

                if (modifiedComplexity.DisplayOrder < 1)
                {
                    updatedItems.Add(modifiedComplexity);
                    return updatedItems;
                }
                bool inLoop = true;
                for (int index = 0; index < dataItems.Count && inLoop; index++)
                {
                    if (dataItems[index].Equals(modifiedComplexity))
                    {
                        dataItems.RemoveAt(index);
                        inLoop = false;
                        break;
                    }
                }
                if (modifiedComplexity.DisplayOrder > totalRecordsCount - 1)
                    dataItems.Add(modifiedComplexity);
                else
                    dataItems.Insert(Convert.ToInt32(modifiedComplexity.DisplayOrder) - 1, modifiedComplexity);
                int orderCounter = 0;

                foreach (Complexity complexity in dataItems)
                {
                    orderCounter++;
                    if (complexity.DisplayOrder != orderCounter || complexity.Equals(modifiedComplexity))
                    {
                        if (!complexity.Equals(modifiedComplexity))
                            complexity.DisplayOrder = Convert.ToUInt32(orderCounter);
                        updatedItems.Add(complexity);
                    }
                }

                return updatedItems;
            }
        }

        #endregion

        #endregion
    }

    #region Other Class
    public class EntityCollection<T> : List<T>, IBindingList, ICloneable where T : EntityObject, new()
    {
        #region Constructors

        public EntityCollection()
        {
        }

        public EntityCollection(List<T> list)
            : base(list)
        {

        }

        public EntityCollection(T[] list)
        {
            foreach (T item in list)
            {
                this.Add(item);
            }
        }

        #endregion

        #region Public Methods

        #region IBindingList Members

        public void AddIndex(PropertyDescriptor property)
        {
            ;
        }

        public virtual object AddNew()
        {
            return new T();
        }

        public bool AllowEdit
        {
            get { return true; }
        }

        public bool AllowNew
        {
            get { return true; }
        }

        public bool AllowRemove
        {
            get { return false; }
        }

        public void ApplySort(PropertyDescriptor property, ListSortDirection direction)
        {
            ;
        }

        public int Find(PropertyDescriptor property, object key)
        {
            return 0;
        }

        public bool IsSorted
        {
            get { return true; }
        }

        public event ListChangedEventHandler ListChanged = null;

        public void RemoveIndex(PropertyDescriptor property)
        {
            ;
        }

        public void RemoveSort()
        {
            ;
        }

        public ListSortDirection SortDirection
        {
            get { return ListSortDirection.Ascending; }
        }

        public PropertyDescriptor SortProperty
        {
            get { return null; }
        }

        public bool SupportsChangeNotification
        {
            get { return false; ; }
        }

        public bool SupportsSearching
        {
            get { return false; }
        }

        public bool SupportsSorting
        {
            get { return false; ; }
        }

        #endregion

        #region ICloneable Members

        public object Clone()
        {
            EntityCollection<T> list = new EntityCollection<T>();
            foreach (T instance in this)
            {
                if (instance is EntityObject)
                    list.Add((T)((EntityObject)instance).Clone());
            }

            return list;
        }

        public void DataChanged()
        {
        }

        #endregion

        public virtual string ToXml()
        {
            StringBuilder XmlString = new StringBuilder();
            XmlString.Append("<ROOT>");
            foreach (EntityObject entityObject in this)
            {
                XmlString.Append(entityObject.GetXml(false));
            }
            XmlString.Append("</ROOT>");
            return XmlString.ToString();
        }

        public override string ToString()
        {
            StringBuilder StringValue = new StringBuilder();
            foreach (EntityObject entityObject in this)
            {
                StringValue.Append(entityObject.ToString()).Append(",");
            }
            return StringValue.ToString();
        }

        //public void Validate(ValidationResults results, InplaceErrorPosition.IEPWindow.Modes operationMode)
        //{
        //    foreach (EntityObject entityObject in this)
        //    {
        //        entityObject.Validate(results, operationMode);
        //    }
        //}

        public void RemoveItems(EntityCollection<T> items)
        {
            RemoveItems(items.ToArray());
        }

        public void RemoveItems(T[] items)
        {
            foreach (T givenItem in items)
            {
                int index = 0;
                foreach (T currentItem in this)
                {
                    if (givenItem.UniqueIdentifier.Equals(currentItem.UniqueIdentifier))
                    {
                        this.RemoveAt(index);
                        break;
                    }
                    index++;
                }
            }

        }

        public void RemoveItems(Guid[] ids)
        {
            foreach (Guid id in ids)
            {
                T searchedItem = this.SearchById(id);
                if (searchedItem != null)
                    this.Remove(searchedItem);
            }
        }

        public void AddItems(EntityCollection<T> items)
        {
            this.AddRange(items);
        }

        public int GetIndexFromGuid(Guid id)
        {
            int i = -1;
            foreach (T item in this)
            {
                if (item.UniqueIdentifier.Equals(id))
                    return i + 1;
                else
                {
                    i = i + 1;
                    continue;
                }
            }
            return -1;
        }

        public T SearchById(Guid id)
        {
            foreach (T item in this)
            {
                if (item != null && item.UniqueIdentifier.Equals(id))
                    return item;
            }
            return null;
        }

        public EntityCollection<T> SearchByIds(Guid[] ids)
        {
            EntityCollection<T> returnList = new EntityCollection<T>();
            foreach (Guid id in ids)
            {
                T searchedItem = this.SearchById(id);
                if (searchedItem != null)
                    returnList.Add(searchedItem);
            }
            return returnList;
        }

        public new bool Contains(T item)
        {
            if (item == null) return false;
            return (SearchById(item.UniqueIdentifier) != null);
        }

        public Guid[] GetGuids()
        {
            List<Guid> items = new List<Guid>(); ;
            foreach (T item in this)
                items.Add(item.UniqueIdentifier);

            return items.ToArray();
        }

        public Guid[] GetGuids(string propertyName)
        {
            List<Guid> items = new List<Guid>(); ;
            if (this.Count < 1)
                return items.ToArray();
            Type tType = this[0].GetType();
            System.Reflection.PropertyInfo[] properties = tType.GetProperties();
            System.Reflection.PropertyInfo property = null;
            foreach (System.Reflection.PropertyInfo propertyItem in properties)
            {
                if (propertyItem.Name == propertyName)
                {
                    property = propertyItem;
                    break;
                }
            }
            if (property == null)
                return items.ToArray();

            foreach (T item in this)
                items.Add((Guid)property.GetValue(item, null));

            return items.ToArray();
        }
        #endregion

    }

    public partial class Complexity : EntityObject
    {
        #region Private Variables

        private System.String _complexityName = string.Empty;
        private System.UInt32 _displayOrder;

        #endregion Private Variables

        #region Public Properties

        public System.String Name
        {
            get { return this._complexityName; }
            set
            {
                if (value != null)
                    this._complexityName = value.Trim();
                else this._complexityName = string.Empty;
            }
        }

        public System.UInt32 DisplayOrder
        {
            get { return this._displayOrder; }
            set { this._displayOrder = value; }
        }

        #endregion Public Properties

        #region Constructor

        public Complexity()
        {
        }

        public Complexity(System.UInt32 displayOrder, System.String complexityName)
        {
            this._displayOrder = displayOrder;
            this._complexityName = complexityName;
        }

        public Complexity(System.Guid complexityId, System.UInt32 displayOrder, System.String complexityName, System.Int32 timestamp)
        {
            base.UniqueIdentifier = complexityId;
            this._displayOrder = displayOrder;
            this._complexityName = complexityName;
            base.Timestamp = timestamp;
        }

        #endregion Constructor

        #region Public Method

        #endregion

    }

    public class EntityObject : ICloneable
    {
        #region Private variables

        private Guid _uniqueIdentifier;
        private int _timestamp;
        private Nullable<DateTime> _modifiedDateTime;

        #endregion

        #region Public Properties

        public Guid UniqueIdentifier
        {
            get { return _uniqueIdentifier; }
            set { _uniqueIdentifier = value; }
        }
        
        public int Timestamp
        {
            get { return _timestamp; }
            set { _timestamp = value; }
        }

        public Nullable<DateTime> ModifiedDateTimeUpdate
        {
            get { return _modifiedDateTime; }
            set { _modifiedDateTime = value; }
        }


        #endregion

        #region Constructors

        public EntityObject()
        {
        }

        public EntityObject(Guid uniqueIdentifier, int timestamp)
        {
            _uniqueIdentifier = uniqueIdentifier;
            _timestamp = timestamp;
        }
        #endregion

        #region Public Methods

        #region ICloneable Members

        public virtual object Clone()
        {
            object newObject = Activator.CreateInstance(this.GetType());
            if (newObject != null)
            {
                PropertyInfo[] properties = newObject.GetType().GetProperties();
                int counter = 0;
                foreach (PropertyInfo property in properties)
                {
                    if (property.CanWrite)
                    {
                        Type ICloneType = property.PropertyType.GetInterface("ICloneable", true);
                        if ((ICloneType != null))
                        {
                            ICloneable IClone = (ICloneable)property.GetValue(this, null);
                            if (IClone != null)
                                properties[counter].SetValue(newObject, IClone.Clone(), null);
                        }
                        else
                        {
                            properties[counter].SetValue(newObject, property.GetValue(this, null), null);
                        }
                    }
                    counter += 1;
                }
            }
            return newObject;
        }

        #endregion


        public override int GetHashCode()
        {
            if (this._uniqueIdentifier.Equals(Guid.Empty))
                return base.GetHashCode();
            else
                return this.UniqueIdentifier.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            if (obj == null) // check if given object is null.
                return false;
            EntityObject target = obj as EntityObject;
            if (target == null)
                return false;
            if (!target.UniqueIdentifier.Equals(_uniqueIdentifier) || !target.Timestamp.Equals(_timestamp))
                return false;
            return true;
        }

        public virtual string GetXml()
        {
            return this.GetXml(true);
        }

        public virtual string GetXml(bool rootNodeRequired)
        {
            StringBuilder xmlString = new StringBuilder();
            StringBuilder xmlStringForList = new StringBuilder();
            if (rootNodeRequired) xmlString.Append("<ROOT>");
            xmlString.Append("<").Append(this.GetType().Name);
            foreach (PropertyInfo propertyInfo in this.GetType().GetProperties())
            {
                if (propertyInfo.PropertyType.Name != "EntityList`1")
                {
                    object propertyValue = propertyInfo.GetValue(this, null);
                    if (propertyValue != null)
                    {
                        xmlString.Append(" " + propertyInfo.Name + " = \"");
                        xmlString.Append(propertyInfo.GetValue(this, null).ToString().Replace(@"\", @"\\").Replace("\"", "\\\""));
                        xmlString.Append("\"");
                    }
                }
                else
                {
                    System.Collections.IEnumerable enumerable = propertyInfo.GetValue(this, null) as System.Collections.IEnumerable;
                    System.Collections.IEnumerator enumerator = enumerable.GetEnumerator();
                    while (enumerator.MoveNext())
                    {
                        EntityObject enityObject = enumerator.Current as EntityObject;
                        if (null != enityObject)
                            xmlStringForList.Append(enityObject.GetXml(false));
                    }
                }
            }
            xmlString.Append(">");
            if (xmlStringForList.Length > 0) xmlString.Append(xmlStringForList.ToString());
            xmlString.Append("</").Append(this.GetType().Name).Append(">");
            if (rootNodeRequired) xmlString.Append("</ROOT>");
            return xmlString.ToString();
        }

        public override string ToString()
        {
            return base.ToString();
        }

        //public virtual void Validate(ValidationResults results, InplaceErrorPosition.IEPWindow.Modes operationMode)
        //{
        //    this.GetValidator(operationMode).Validate(this, results);
        //}

        //virtual protected Validator GetValidator(InplaceErrorPosition.IEPWindow.Modes mode)
        //{
        //    string ruleSet;
        //    switch (mode)
        //    {
        //        case InplaceErrorPosition.IEPWindow.Modes.Add:
        //            ruleSet = "RuleSetAdd";
        //            break;
        //        default:
        //            ruleSet = "RuleSetEdit";
        //            break;
        //    }
        //    return ValidationFactory.CreateValidatorFromAttributes(this.GetType(), ruleSet);
        //}
        #endregion

    }

    public class FaultRecords
    {
        #region variables
        private Record _faultRecords;
        private string _faultMessage = string.Empty;
        private bool _Isfaulted = true;
        #endregion

        #region properties
        public Record ErrorRecord
        {
            get
            {
                return _faultRecords;
            }
            set
            {
                _faultRecords = value;
            }
        }

        public string FaultMessage
        {
            get
            {
                return _faultMessage;
            }
            set
            {
                _faultMessage = value;
            }
        }

        public bool ISFaulted
        {
            get
            {
                return _Isfaulted;
            }
        }
        #endregion

        #region constructor
        public FaultRecords(Record faultRecord, string faultMessage)
        {
            _faultMessage = faultMessage;
            _faultRecords = faultRecord;
        }

        internal FaultRecords(bool isfaulted, string faultMessage)
        {
            _Isfaulted = isfaulted;
            _faultMessage = faultMessage;
        }
        #endregion
    }
    #endregion
}
